using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Windows.Forms;
using ivware.Mobile.RAPISync.Store;
using OpenNETCF.Desktop.Communication;

namespace ivware.Mobile.RAPISync.GUI.Controls
{
	/// <summary> class DeviceFolderSelect 
	/// <para>An example on how to build a folder browser dialog window using C# and the .Net framework.</para>
	/// </summary>
	public class DeviceFolderSelect : Form
	{
		private string m_SelectedFolder;
		private Panel panel3;
		private TreeView DeviceFolderTree;
		private Button cancelBtn;
		private Panel panel2;
		private Button selectBtn;
		private ImageList DeviceImageList;
		private Button MakeNewFolder;
		private IContainer components;

		public DeviceFolderSelect()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// initialize the treeView
			LoadTree();
		}

		#region Private Methods
		
		private void LoadTree()
		{
			// clear out the old values
			DeviceFolderTree.Nodes.Clear();

			TreeNode newNode = new TreeNode("Mobile Device");
			newNode.Tag = @"\";
			newNode.ImageIndex = 3;
			DeviceFolderTree.Nodes.Add(newNode); // add the new node to the root level.
			GetSubFolders(newNode); // scan for any sub folders on this drive.
			newNode.Expand();
		}

		private void GetSubFolders(TreeNode parent)
		{
			Cursor.Current = Cursors.WaitCursor;

			try
			{
				if (parent.Nodes.Count == 1 && parent.Nodes[0].Text == string.Empty)
					parent.Nodes.Clear(); //remove the dummy node

				if (parent.Nodes.Count == 0)
				{
					List<FileInformation> children = GetDirectories(parent.Tag as String);
					foreach (FileInformation f in children)
					{
						if ((f.FileAttributes & (int)RAPI.RAPIFileAttributes.Hidden) > 0
								|| (f.FileAttributes & (int)RAPI.RAPIFileAttributes.System) > 0)
							continue;
						
						TreeNode newNode = new TreeNode(f.FileName);
						newNode.Tag = Path.Combine(parent.Tag as String, f.FileName);

						parent.Nodes.Add(newNode);
						
						if ((f.FileAttributes & (int)RAPI.RAPIFileAttributes.Temporary) > 0
								&& (f.FileAttributes & (int)RAPI.RAPIFileAttributes.Directory) > 0)
						{
							newNode.ImageIndex = 2;
							newNode.Expand();
						}

						newNode.Nodes.Add(new TreeNode("")); //insert a dummy node
					}
				}
			}
			finally
			{
				Cursor.Current = Cursors.Default;
			}
		}

		private List<FileInformation> GetDirectories(string i_path)
		{
			List<FileInformation> retVal = new List<FileInformation>();

			if (!RAPIUtils.DeviceConnected)
			{
				MessageBox.Show("Mobile device is not connected!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error,
				                MessageBoxDefaultButton.Button1);
				return new List<FileInformation>();
			}

			if (!RAPIUtils.Instance.RAPI.Connected)
			{
				RAPIUtils.Instance.RAPI.Connect();
			}

			if (RAPIUtils.Instance.RAPI.DeviceFileExists(i_path))
			{
				// Process the directory list
				FileList files = RAPIUtils.Instance.RAPI.EnumFiles(Path.Combine(i_path, "*.*"));
				if (files != null)
				{
					foreach (FileInformation entry in files)
					{
						if ((entry.FileAttributes & (int) RAPI.RAPIFileAttributes.Directory) > 0)
							retVal.Add(entry);
					}
				}
			}

			return retVal;
		}

		#endregion

		#region Windows Form Designer generated code

		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.components = new System.ComponentModel.Container();
			System.Windows.Forms.TreeNode treeNode1 = new System.Windows.Forms.TreeNode("Node2");
			System.Windows.Forms.TreeNode treeNode2 = new System.Windows.Forms.TreeNode("Node1", new System.Windows.Forms.TreeNode[] {
            treeNode1});
			System.Windows.Forms.TreeNode treeNode3 = new System.Windows.Forms.TreeNode("Node0", new System.Windows.Forms.TreeNode[] {
            treeNode2});
			System.ComponentModel.ComponentResourceManager resources = new System.ComponentModel.ComponentResourceManager(typeof(DeviceFolderSelect));
			this.panel2 = new System.Windows.Forms.Panel();
			this.MakeNewFolder = new System.Windows.Forms.Button();
			this.cancelBtn = new System.Windows.Forms.Button();
			this.selectBtn = new System.Windows.Forms.Button();
			this.panel3 = new System.Windows.Forms.Panel();
			this.DeviceFolderTree = new System.Windows.Forms.TreeView();
			this.DeviceImageList = new System.Windows.Forms.ImageList(this.components);
			this.panel2.SuspendLayout();
			this.panel3.SuspendLayout();
			this.SuspendLayout();
			// 
			// panel2
			// 
			this.panel2.AutoScroll = true;
			this.panel2.Controls.Add(this.MakeNewFolder);
			this.panel2.Controls.Add(this.cancelBtn);
			this.panel2.Controls.Add(this.selectBtn);
			this.panel2.Dock = System.Windows.Forms.DockStyle.Bottom;
			this.panel2.Location = new System.Drawing.Point(6, 304);
			this.panel2.Name = "panel2";
			this.panel2.Padding = new System.Windows.Forms.Padding(10);
			this.panel2.Size = new System.Drawing.Size(388, 48);
			this.panel2.TabIndex = 1;
			// 
			// MakeNewFolder
			// 
			this.MakeNewFolder.Location = new System.Drawing.Point(13, 12);
			this.MakeNewFolder.Name = "MakeNewFolder";
			this.MakeNewFolder.Size = new System.Drawing.Size(136, 23);
			this.MakeNewFolder.TabIndex = 4;
			this.MakeNewFolder.Text = "&Make New Device Folder";
			this.MakeNewFolder.UseVisualStyleBackColor = true;
			this.MakeNewFolder.Click += new System.EventHandler(this.OnMakeNewFolderClick);
			// 
			// cancelBtn
			// 
			this.cancelBtn.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.cancelBtn.DialogResult = System.Windows.Forms.DialogResult.Cancel;
			this.cancelBtn.Location = new System.Drawing.Point(303, 13);
			this.cancelBtn.Name = "cancelBtn";
			this.cancelBtn.Size = new System.Drawing.Size(75, 23);
			this.cancelBtn.TabIndex = 3;
			this.cancelBtn.Text = "&Cancel";
			this.cancelBtn.Click += new System.EventHandler(this.OnCancelClick);
			// 
			// selectBtn
			// 
			this.selectBtn.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
			this.selectBtn.DialogResult = System.Windows.Forms.DialogResult.OK;
			this.selectBtn.Location = new System.Drawing.Point(218, 13);
			this.selectBtn.Name = "selectBtn";
			this.selectBtn.Size = new System.Drawing.Size(79, 23);
			this.selectBtn.TabIndex = 2;
			this.selectBtn.Text = "&OK";
			this.selectBtn.Click += new System.EventHandler(this.OnSelectClick);
			// 
			// panel3
			// 
			this.panel3.Controls.Add(this.DeviceFolderTree);
			this.panel3.Dock = System.Windows.Forms.DockStyle.Fill;
			this.panel3.Location = new System.Drawing.Point(6, 6);
			this.panel3.Name = "panel3";
			this.panel3.Size = new System.Drawing.Size(388, 298);
			this.panel3.TabIndex = 2;
			// 
			// DeviceFolderTree
			// 
			this.DeviceFolderTree.BackColor = System.Drawing.Color.LightSteelBlue;
			this.DeviceFolderTree.Dock = System.Windows.Forms.DockStyle.Fill;
			this.DeviceFolderTree.ImageIndex = 0;
			this.DeviceFolderTree.ImageList = this.DeviceImageList;
			this.DeviceFolderTree.ImeMode = System.Windows.Forms.ImeMode.NoControl;
			this.DeviceFolderTree.Location = new System.Drawing.Point(0, 0);
			this.DeviceFolderTree.Name = "DeviceFolderTree";
			treeNode1.Name = "";
			treeNode1.Text = "Node2";
			treeNode2.Name = "";
			treeNode2.Text = "Node1";
			treeNode3.Name = "";
			treeNode3.Text = "Node0";
			this.DeviceFolderTree.Nodes.AddRange(new System.Windows.Forms.TreeNode[] {
            treeNode3});
			this.DeviceFolderTree.SelectedImageIndex = 0;
			this.DeviceFolderTree.ShowRootLines = false;
			this.DeviceFolderTree.Size = new System.Drawing.Size(388, 298);
			this.DeviceFolderTree.TabIndex = 0;
			this.DeviceFolderTree.BeforeExpand += new System.Windows.Forms.TreeViewCancelEventHandler(this.OnBeforeExpand);
			this.DeviceFolderTree.AfterLabelEdit += new System.Windows.Forms.NodeLabelEditEventHandler(this.OnAfterEdit);
			this.DeviceFolderTree.BeforeSelect += new System.Windows.Forms.TreeViewCancelEventHandler(this.OnBeforeSelect);
			// 
			// DeviceImageList
			// 
			this.DeviceImageList.ImageStream = ((System.Windows.Forms.ImageListStreamer)(resources.GetObject("DeviceImageList.ImageStream")));
			this.DeviceImageList.TransparentColor = System.Drawing.Color.Teal;
			this.DeviceImageList.Images.SetKeyName(0, "folder_closed_sm.bmp");
			this.DeviceImageList.Images.SetKeyName(1, "folder_sm.bmp");
			this.DeviceImageList.Images.SetKeyName(2, "storage_cd_sm.bmp");
			this.DeviceImageList.Images.SetKeyName(3, "pda_sm.bmp");
			// 
			// DeviceFolderSelect
			// 
			this.AcceptButton = this.selectBtn;
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.CancelButton = this.cancelBtn;
			this.ClientSize = new System.Drawing.Size(400, 358);
			this.Controls.Add(this.panel3);
			this.Controls.Add(this.panel2);
			this.MaximizeBox = false;
			this.MinimizeBox = false;
			this.MinimumSize = new System.Drawing.Size(230, 300);
			this.Name = "DeviceFolderSelect";
			this.Padding = new System.Windows.Forms.Padding(6);
			this.ShowIcon = false;
			this.ShowInTaskbar = false;
			this.SizeGripStyle = System.Windows.Forms.SizeGripStyle.Show;
			this.StartPosition = System.Windows.Forms.FormStartPosition.CenterParent;
			this.Text = "Browse for Device Folder";
			this.panel2.ResumeLayout(false);
			this.panel3.ResumeLayout(false);
			this.ResumeLayout(false);

		}

		#endregion

		#region UI Event Handlers

		private void OnBeforeSelect(object sender, TreeViewCancelEventArgs e)
		{
			m_SelectedFolder = e.Node.Tag as String;
		}

		private void OnBeforeExpand(object sender, TreeViewCancelEventArgs e)
		{
			GetSubFolders(e.Node); // get the sub-folders for the selected node.
			m_SelectedFolder = e.Node.Tag as String;
			if(e.Node.ImageIndex == 0)
				e.Node.ImageIndex = 1;
		}

		private void OnCancelClick(object sender, EventArgs e)
		{
			m_SelectedFolder = null;
			Close();
		}

		private void OnAfterEdit(object sender, NodeLabelEditEventArgs e)
		{
			if (e.Node.Tag == null)
			{
				//Cancel
				if (e.Label == null)
				{
					e.Node.Remove();
					return;
				}

				string path = Path.Combine(e.Node.Parent.Tag as string, e.Label);

				if (!RAPIUtils.Instance.RAPI.DeviceFileExists(path))
				{
					try
					{
						RAPIUtils.Instance.RAPI.CreateDeviceDirectory(path);
						e.Node.Tag = path;
						DeviceFolderTree.SelectedNode = e.Node;
					}
					catch
					{
						MessageBox.Show("Unable to create device folder", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error,
						                MessageBoxDefaultButton.Button1);
						e.Node.BeginEdit();
						return;
					}
				}
				else
				{
					MessageBox.Show("Folder with this name already exists", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error,
					                MessageBoxDefaultButton.Button1);
					e.Node.BeginEdit();
					return;
				}
			}

			DeviceFolderTree.LabelEdit = false;
		}

		private void OnMakeNewFolderClick(object sender, EventArgs e)
		{
			TreeNode newNode = new TreeNode("New Folder");
			DeviceFolderTree.SelectedNode.Nodes.Add(newNode);
			DeviceFolderTree.SelectedNode.Expand();
			DeviceFolderTree.LabelEdit = true;
			newNode.BeginEdit();
		}

		private void OnSelectClick(object sender, EventArgs e)
		{
			DialogResult = DialogResult.OK;
			Close();
		}

		#endregion

		#region Properties

		public string SelectedFolder
		{
			get { return m_SelectedFolder; }
		}

		#endregion

	}
}