using System;
using System.Diagnostics;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Threading;
using System.Windows.Forms;

namespace ivware.Mobile.RAPISync.GUI
{
	public class SingleInstance : IDisposable
	{
		//Win32 API calls necesary to raise an unowned processs main window
		[DllImport("user32.dll")]
		private static extern bool SetForegroundWindow(IntPtr hWnd);

		[DllImport("user32.dll")]
		private static extern bool ShowWindowAsync(IntPtr hWnd, int nCmdShow);

		[DllImport("user32.dll")]
		private static extern bool IsIconic(IntPtr hWnd);

		[DllImport("user32.dll", SetLastError = true)]
		static extern bool PostMessage(IntPtr hWnd, uint Msg, IntPtr wParam, IntPtr lParam);

		private IntPtr HWND_BROADCAST = (IntPtr)65535;
		
		[DllImport("user32.dll")]
		private static extern int RegisterWindowMessage( string lpString );

		private const int SW_RESTORE = 9;

		//private members 
		private Mutex _processSync;
		private bool _owned = false;
		static uint m_ActivateMessage = 0;
		
		public SingleInstance() : this("") {}

		public SingleInstance(string identifier)
		{
			string applicationName = System.Reflection.Assembly.GetExecutingAssembly().GetName().Name;
			
			_processSync = new Mutex(
				true, // desire intial ownership
				applicationName + identifier,
				out _owned);
		}

		public static uint ActivateMessage
		{
			get
			{
				if (m_ActivateMessage == 0)
				{
					string applicationName = System.Reflection.Assembly.GetExecutingAssembly().GetName().Name;
					m_ActivateMessage = (uint)RegisterWindowMessage(applicationName);
				}

				return m_ActivateMessage;
			}
		}
		
		~SingleInstance()
		{
			//Release mutex (if necessary) 
			//This should have been accomplished using Dispose() 
			Release();
		}

		public bool IsSingleInstance
		{
			//If we don't own the mutex than
			// we are not the first instance.
			get { return _owned; }
		}
		
		public void NotifyRunningProcess(IntPtr hWindow)
		{
			PostMessage(hWindow,
								 ActivateMessage,
								 IntPtr.Zero,
								 IntPtr.Zero);
		}

		public void RaiseOtherProcess()
		{
			Process proc = Process.GetCurrentProcess();
			// Using Process.ProcessName does not function properly when
			// the actual name exceeds 15 characters. Using the assembly 
			// name takes care of this quirk and is more accruate than 
			// other work arounds.
			string assemblyName =
				Assembly.GetExecutingAssembly().GetName().Name;
			foreach (Process otherProc in 
				Process.GetProcessesByName(assemblyName))
			{
				//ignore "this" process
				if (proc.Id != otherProc.Id)
				{
					// Found a "same named process".
					// Assume it is the one we want brought to the foreground.
					// Use the Win32 API to bring it to the foreground.
					IntPtr hWnd = otherProc.MainWindowHandle;
					
					NotifyRunningProcess(hWnd);
					
					/*if (IsIconic(hWnd))
					{
						ShowWindowAsync(hWnd, SW_RESTORE);
					}
					SetForegroundWindow(hWnd);*/
					break;
				}
			}
		}

		private void Release()
		{
			if (_owned)
			{
				//If we own the mutex than release it so that
				// other "same" processes can now start.
				_processSync.ReleaseMutex();
				_owned = false;
			}
		}

		#region Implementation of IDisposable

		public void Dispose()
		{
			//release mutex (if necessary) and notify 
			// the garbage collector to ignore the destructor
			Release();
			GC.SuppressFinalize(this);
		}

		#endregion
	}
}