using System;
using System.IO;
using ivware.Mobile.RAPISync.Core;
using OpenNETCF.Desktop.Communication;

namespace ivware.Mobile.RAPISync.Store
{
	/// <summary>
	/// Represents an item in the CE File system
	/// </summary>
	public class CEStoreItem : StoreItem
	{
		private FileInformation m_MyInfo = null;
		private string m_ParentPath;

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="i_path"></param>
		public CEStoreItem(string i_path)
		{
			m_ParentPath = i_path;
		}

		public CEStoreItem(string i_path, CEStore i_Store) : this(i_path)
		{
			if (RAPIUtils.Instance.RAPI.DeviceFileExists(i_path))
			{
				// Process the directory list
				FileList files = RAPIUtils.Instance.RAPI.EnumFiles(Path.Combine(i_path, "."));
				if (files != null)
				{
					foreach (FileInformation entry in files)
					{
						m_MyInfo = entry;
						m_ItemStore = i_Store;
						m_ParentPath = Path.GetDirectoryName(i_path);
					}
				}
			}
			else
			{
				m_ItemStore = i_Store;
			}
		}

		private CEStoreItem()
		{
		}

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="i_info"></param>
		public CEStoreItem(FileInformation i_info, string i_ParentPath, CEStore i_Store)
		{
			m_MyInfo = i_info;
			m_ItemStore = i_Store;
			m_ParentPath = i_ParentPath;
		}

		/// <summary>
		/// The path to the item
		/// </summary>
		public override string ItemPath
		{
			get
			{
				if (m_MyInfo == null)
					return m_ParentPath;
				else
					return Path.Combine(m_ParentPath, m_MyInfo.FileName);
				;
			}
		}

		public override object NativeItem
		{
			get { return m_MyInfo; }
		}

		public override string ItemRelativePath
		{
			get
			{
				if (m_ItemStore != null)
				{
					return ((CEStore) m_ItemStore).CalculateRelativePath(this);
				}
				else
					return ItemPath;
			}
		}

		public override bool IsPlaceholder
		{
			get { return m_MyInfo == null; }
		}

		public override DateTime LastWriteTime
		{
			get
			{
				if (m_MyInfo != null)
					return m_MyInfo.LastWriteTime;
				else
					return base.LastWriteTime;
			}
		}

		/// <summary>
		/// Checks that the item exists on the device
		/// </summary>
		public override bool Exists
		{
			get
			{
					if (!RAPIUtils.Instance.RAPI.DeviceFileExists(ItemPath))
						return false;

					return true;
			}
		}

		/// <summary>
		/// Size of the file in bytes
		/// </summary>
		public override long Size
		{
			get
			{
				if (NativeItem == null || IsDirectory)
					return base.Size;
				else
					return ((FileInformation) NativeItem).FileSize;
			}
		}

		/// <summary>
		/// True if the item represents a directory
		/// </summary>
		public override bool IsDirectory
		{
			get
			{
				if (IsPlaceholder)
					return base.IsDirectory;

				if ((m_MyInfo.FileAttributes & (int) RAPI.RAPIFileAttributes.Directory) > 0)
					return true;
				else
					return false;
			}
		}

		public override bool IsReadOnly
		{
			get
			{
				if (IsPlaceholder)
					return true;

				if ((m_MyInfo.FileAttributes & (int) RAPI.RAPIFileAttributes.InROM) > 0)
					return true;

				if ((m_MyInfo.FileAttributes & (int) RAPI.RAPIFileAttributes.ReadOnly) > 0)
					return true;

				return false;
			}
		}
	}
}