using System;

namespace ivware.Mobile.RAPISync.Core
{
	public class ItemDiffGram
	{
		public enum SyncAction
		{
			CopyToSource,
			CopyToDestination,
			DeleteFromDestination,
			Unchanged,
			Ignore
		} ;

		private StoreItem m_Source;
		private StoreItem m_Destination;
		private SyncAction m_Action;
		private OperationResult m_Result = new OperationResult();

		/// <summary>
		/// Constructor
		/// </summary>
		/// <param name="i_Source"></param>
		/// <param name="i_Dest"></param>
		/// <param name="i_Rule"></param>
		public ItemDiffGram(StoreItem i_Source, StoreItem i_Dest, SyncRule i_Rule)
		{
			m_Source = i_Source;
			m_Destination = i_Dest;

			if (m_Source.Equals(m_Destination))
			{
				m_Action = SyncAction.Unchanged;
				return;
			}

			//  Sync to left  S ---> D
			if (i_Rule.Rule == SyncRule.RuleType.SourceOverridesDestination ||
			    i_Rule.Rule == SyncRule.RuleType.SourceContributesToDestination)
			{
				// 1 --- 1, 1 --- 0
				if (!m_Source.IsPlaceholder)
					m_Action = SyncAction.CopyToDestination;
				else
					// 0 --- 1
					if (m_Source.IsPlaceholder && !m_Destination.IsPlaceholder)
						m_Action = SyncAction.DeleteFromDestination;
					else
						throw new ApplicationException(" Both Items can't be placeholders! ");
			}
				// Sync Both Ways S <---> D
			else
			{
				// 1 --- 1 - Use CompareTo to decide 
				if (!m_Source.IsPlaceholder && !m_Destination.IsPlaceholder)
				{
					if (m_Source.CompareTo(m_Destination) == StoreItem.CompareResult.NewerCopyOfTheSameFile)
						m_Action = SyncAction.CopyToDestination;
					else
						m_Action = SyncAction.CopyToSource;
				}
				else
					// 0 --- 1
					if (m_Source.IsPlaceholder && !m_Destination.IsPlaceholder)
						m_Action = SyncAction.CopyToSource;

					else
						// 1 --- 0   
						if (!m_Source.IsPlaceholder && m_Destination.IsPlaceholder)
							m_Action = SyncAction.CopyToDestination;
						else
							throw new ApplicationException(" Both Items can't be placeholders! ");
			}
		}

		public override string ToString()
		{
			return string.Format("({2}) {0} <###> {1} - Result: {3}", m_Source, m_Destination, m_Action, m_Result);
		}

		/// <summary>
		/// Action to be performed
		/// </summary>
		public SyncAction Action
		{
			get { return m_Action; }
			internal set { m_Action = value; }
		}

		/// <summary>
		/// Source Item for the Operation
		/// </summary>
		public StoreItem Source
		{
			get { return m_Source; }
		}

		/// <summary>
		/// Destination Item for the operation
		/// </summary>
		public StoreItem Destination
		{
			get { return m_Destination; }
		}

		/// <summary>
		/// The result of the operation
		/// </summary>
		public OperationResult Result
		{
			get { return m_Result; }
			set { m_Result = value; }
		}
	}
}