using System;
using System.Collections.Generic;
using System.Text;

namespace ivware.Mobile.RAPISync.Core
{
  /// <summary>
  /// Describes the result of the operation
  /// </summary>
  public class OperationResult
  {
    #region Data Members
    public enum ResultStatus
    {
      NotStarted,
      Success,
      Failed,
      Canceled,
      Ignored
    }

    private ResultStatus m_Status = ResultStatus.NotStarted;
    private string m_ReasonFailed = string.Empty;
    private Exception m_LastException;
    #endregion

    #region Public Properties
    /// <summary>
    /// Teh status of the operation
    /// </summary>
    public ResultStatus Status
    {
      get
      {
        return m_Status;
      }
      set
      {
        m_Status = value;
      }
    }

    /// <summary>
    /// COntains the exception that was thrown 
    /// </summary>
    public string ReasonFailed
    {
      get
      {
        return m_ReasonFailed;
      }
      set
      {
        m_ReasonFailed = value;
      }
    }

    /// <summary>
    /// The last exception that was thrown while attempting the operation
    /// </summary>
    public Exception LastException
    {
      get
      {
        return m_LastException;
      }
      set
      {
        m_LastException = value;
        Status = ResultStatus.Failed;
        ReasonFailed = value.Message;
      }
    }
    #endregion

    #region Constructors
    /// <summary>
    /// Constructor
    /// </summary>
    /// <param name="i_Status"></param>
    public OperationResult()
    {
      Status= ResultStatus.NotStarted;
    }

    #endregion

    #region Public Methods

    public override string ToString()
    {
      switch (Status)
      {
        case ResultStatus.Canceled:
          return "Canceled";

        case ResultStatus.Failed:
          return "Failed! Reason: " + ReasonFailed;

        case ResultStatus.NotStarted:
          return "Waiting to Start";

        case ResultStatus.Success:
          return "Success";

        case ResultStatus.Ignored:
          return "Ignored";

        default:
          return "Unknown";
      }
    }
    #endregion
  }
}
