using System;
using System.Collections.Generic;
using System.Text;
using ivware.Mobile.RAPISync.Store;
using System.IO;

namespace ivware.Mobile.RAPISync.Core
{
  /// <summary>
  /// Constructs File Stores at run time
  /// </summary>
  public class StoreFactory
  {
    

    public enum Store
    {
      FileSystem,
      CEStore
    }

    /// <summary>
    /// Creates a new Store instance
    /// </summary>
    /// <param name="i_Store"></param>
    /// <returns></returns>
    public static IFileStore CreateStore(Store i_Store)
    {
      switch (i_Store)
      {
        case Store.CEStore:
          return new CEStore();

        case Store.FileSystem:
          return new FileSystemStore();

        default:
          throw new ArgumentException("Invalid Store Type Provided!");
      }
    }

    /// <summary>
    /// Extracts a Store type and Path from a url
    /// </summary>
    /// <param name="i_Url"></param>
    /// <param name="i_path"></param>
    /// <returns></returns>
    public static Store SplitUrl(string i_Url, out string i_path)
    {
      if (i_Url.StartsWith(CEStore.STORE_CEFILE, false, null))
      {
				i_path = i_Url.Substring(CEStore.STORE_CEFILE.Length);
        return Store.CEStore;
      }
      else
      {
        i_path = i_Url;
        return Store.FileSystem;
      }
    }
  }
}
