using System;
using System.IO;
using ivware.Mobile.RAPISync.Core;

namespace ivware.Mobile.RAPISync.Store
{
  /// <summary>
  /// Represents an item in the CE File system
  /// </summary>
  public class FileSystemStoreItem : StoreItem
  {
    private FileSystemInfo m_MyInfo = null;
    private DriveInfo m_MyDrive;

    /// <summary>
    /// Constructor
    /// </summary>
    /// <param name="i_path"></param>
    public FileSystemStoreItem(string i_path, FileSystemStore i_Store) : this(i_path)
    {
      m_ItemStore = i_Store;
    }

    public FileSystemStoreItem(string i_path)
    {
      m_MyInfo = new DirectoryInfo(i_path);
    }

    private FileSystemStoreItem()
    {
    }

    /// <summary>
    /// Constructor
    /// </summary>
    /// <param name="i_info"></param>
    public FileSystemStoreItem(FileSystemInfo i_info, FileSystemStore i_Store)
    {
      m_MyInfo = i_info;
      m_ItemStore = i_Store;
    }

    /// <summary>
    /// The drive on which this item resides
    /// </summary>
    public DriveInfo Drive
    {
      get
      {
        string root = Path.GetPathRoot(ItemPath);
        
        if (m_MyDrive == null)
        {
          DriveInfo[] drives = DriveInfo.GetDrives();

          foreach (DriveInfo d in drives)
          {
            if (string.Compare(Path.GetPathRoot(d.RootDirectory.FullName), root, true) == 0)
            {
              m_MyDrive = d;
              break;
            }
          }
        }
        
        return m_MyDrive;
      }
    }
    
    public override bool IsPlaceholder
    {
      get
      {
        if (m_MyInfo == null || m_MyInfo.Exists == false)
          return true;
        else
          return false;
      }
    }

    /// <summary>
    /// The path to the item
    /// </summary>
    public override string ItemPath
    {
      get { return m_MyInfo.FullName; }
    }

    public override object NativeItem
    {
      get { return m_MyInfo; }
    }

    public override string ItemRelativePath
    {
      get
      {
        if (m_ItemStore != null)
        {
          return ((FileSystemStore) m_ItemStore).CalculateRelativePath(this);
        }
        else
          return ItemPath;
      }
    }

    public override DateTime LastWriteTime
    {
      get
      {
        if (m_MyInfo != null)
          return m_MyInfo.LastWriteTime;
        else
          return base.LastWriteTime;
      }
    }

    public override bool IsDirectory
    {
      get
      {
        if (!IsPlaceholder)
          return ((m_MyInfo.Attributes & FileAttributes.Directory) > 0);

        return base.IsDirectory;
      }
    }

    /// <summary>
    /// Size of the file in bytes
    /// </summary>
    public override long Size
    {
      get
      {
        if (NativeItem == null || IsDirectory)
          return base.Size;
        else
        {
          return ((FileInfo) NativeItem).Length;
        }
      }
    }

    public override bool Exists
    {
      get
      {
        if (!IsPlaceholder)
          return m_MyInfo.Exists;
        else
          return base.Exists;
      }
    }

    public override bool IsReadOnly
    {
      get
      {
        if (NativeItem == null)
          return true;

				if (IsDirectory)
					return false;
				else
					return ((FileInfo)NativeItem).IsReadOnly;
      }
    }
  }
}