using System;
using System.Diagnostics;
using System.IO;
using ivware.Mobile.RAPISync.Store;
using OpenNETCF.Desktop.Communication;

namespace ivware.Mobile.RAPISync.UnitTests
{
  /// <summary>
  /// Summary description for MockDataBuilder.
  /// </summary>
  public class MockDataBuilder
  {
    private static string ZipDir
    {
      get { return Path.GetDirectoryName(Path.GetDirectoryName(Environment.CurrentDirectory)); }
    }

    private static string MockDir
    {
      get { return Path.Combine(ZipDir, "Folder 1"); }
    }

    public static string RAPIMockDir
    {
      get
      {
        return Path.Combine(RAPITestDir, "Folder 1");
        ;
      }
    }

    public static string RAPITestDir
    {
      get { return @"\Storage card\TestTemp"; }
    }

    public static string BuildRAPIMockData()
    {
      BuildPCMockData();
      DeleteRAPIFile(RAPIMockDir);

      //Create the test directory
      if (!RAPIUtils.ItemExists(new CEStoreItem(TestData.CE_TEST_ROOT)))
        RAPIUtils.CreateRAPIFolder(new CEStoreItem(TestData.CE_TEST_ROOT));

      Execute(Path.Combine(ZipDir, "rcmd.exe"),
              " copy \"" + MockDir + "\" \"" + Path.GetDirectoryName(RAPIMockDir) + "\"");
      return RAPIMockDir;
    }

    public static void DeleteRAPIFile(string i_startDirectory)
    {
      Execute(Path.Combine(ZipDir, "rcmd.exe"), " del \"" + i_startDirectory + "\"");
    }

    public static string BuildPCMockData()
    {
      if (Directory.Exists(MockDir))
        Directory.Delete(MockDir, true);

      Execute(Path.Combine(ZipDir, "PCMockFolders.exe"), " x -o " + ZipDir + "\\ *.*");

      if (!Directory.Exists(MockDir))
        throw new ApplicationException("Unable to create PC mock data: " + MockDir);

      return MockDir;
    }

    private static int Execute(string i_exe, string i_args)
    {
      Process p = new Process();

      p.StartInfo = new ProcessStartInfo(i_exe, i_args);
      p.Start();
      p.WaitForExit(10000);
      return p.ExitCode;
    }
  }
}