using ivware.Mobile.RAPISync.Core;
using ivware.Mobile.RAPISync.Store;
using NUnit.Framework;

namespace ivware.Mobile.RAPISync.UnitTests
{
	[Ignore]
  [TestFixture]
  public class TestCEStore
  {
    [TestFixtureSetUp]
    public void FixtureSetUp()
    {
      TestUtilities_CE.CleanupCEFolder(TestData.CE_TEST_ROOT + @"\Folder XX");
      TestUtilities_CE.CleanupCEFolder(TestData.CE_TEST_ROOT + @"\Folder X");
      TestUtilities_CE.CleanupCEFolder(TestData.CE_TEST_ROOT + @"\Folder Y");
    }

    [SetUp]
    public void SetUp()
    {
    }

    [TearDown]
    public void TearDown()
    {
    }

    [TestFixtureTearDown]
    public void FixtureTearDown()
    {
    }

    [Test]
    public void TestCERunProfile()
    {
      SyncEndPoint src = new SyncEndPoint(TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1", StoreFactory.Store.FileSystem);
      SyncEndPoint dst = new SyncEndPoint(TestData.CE_TEST_ROOT_URL + @"\Folder X");
      ProfileTester.TestRunProfile(src, dst);
    }
    
    [Test]
    public void TestSyncPoint()
    {
      SyncEndPoint s = new SyncEndPoint(TestData.CE_TEST_ROOT_URL + @"\Folder X");
      IFileStore store = s.GetStore();
      Assert.IsTrue(store is CEStore);
    }
    
    [Test]
    public void TestStore()
    {
      CEStoreItem i = new CEStoreItem(TestData.CE_TEST_ROOT + @"\Folder 1");
      CEStore store = new CEStore();
      store.InitializeRoot(i);
      Assert.AreEqual(9, store.Items.Count);
    }

    public void TestBatchProfile()
    {
      string profile = TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1\Folder 1-1,";
      profile += TestData.CE_TEST_ROOT_URL + @"\Folder Y\Folder 1-1,";
      profile += " -o";
      profile += "\n";
      profile += TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1\Folder 1-2, ";
      profile += TestData.CE_TEST_ROOT_URL + @"\Folder Y\Folder 1-2, ";
      profile += " -o";

      BatchProfile bp = new BatchProfile(profile);
      Assert.AreEqual(2, bp.Profiles.Count);

      Assert.AreEqual(2, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(2, bp.Profiles[1].Diffgram.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);

			bp.ExecuteAllProfiles();

      Assert.AreEqual(Profile.State.Completed, bp.Profiles[0].ProfileState);
      Assert.AreEqual(Profile.State.Completed, bp.Profiles[1].ProfileState);
      Assert.AreEqual(true, bp.Profiles[0].MadeChanges);
      Assert.AreEqual(true, bp.Profiles[1].MadeChanges);

      //Now delete
      profile = TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder X,";
      profile += TestData.CE_TEST_ROOT_URL + @"\Folder Y,";
      profile += " -o";

      bp = new BatchProfile(profile);
      Assert.AreEqual(1, bp.Profiles.Count);

      Assert.AreEqual(4, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.DeleteFromDestination).Count);

			bp.ExecuteAllProfiles();

      Assert.AreEqual(Profile.State.Completed, bp.Profiles[0].ProfileState);
      Assert.AreEqual(true, bp.Profiles[0].MadeChanges);

    }

    [Test]
    public void TestFileCopyOnce()
    {
      string profile = TestData.FILE_SYSTEM_TEST_ROOT + @"\Folder 1,";
      profile += TestData.CE_TEST_ROOT_URL + @"\Folder Y,";
      profile += " -o,";
      profile += " ce_test,";
      profile += " -h";

      BatchProfile bp = new BatchProfile(profile);
      Assert.AreEqual(1, bp.Profiles.Count);
      Assert.IsTrue(bp.Profiles[0].CopyOnce);

      bp.Profiles[0].CleanHistory();

      Assert.AreEqual(8, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);

      bp.ExecuteAllProfiles();

      Assert.AreEqual(8, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Success).Count);
      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Ignored).Count);

      TestUtilities_CE.CleanupCEFolder(TestData.CE_TEST_ROOT + @"\Folder Y");

      bp.Profiles[0].Refresh();

      Assert.AreEqual(8, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Ignore).Count);
      Assert.AreEqual(1, bp.Profiles[0].Diffgram.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);

      bp.ExecuteAllProfiles();

      Assert.AreEqual(0, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Success).Count);
      Assert.AreEqual(9, bp.Profiles[0].Diffgram.GetByResult(OperationResult.ResultStatus.Ignored).Count);
    }
    
    [Test]
    public void TestCEDiffGram()
    {
      CEStoreItem src = new CEStoreItem(TestData.CE_TEST_ROOT + @"\Folder 1");
      IFileStore src_store = StoreFactory.CreateStore(StoreFactory.Store.CEStore);
      src_store.InitializeRoot(src);

      CEStoreItem dest = new CEStoreItem(TestData.CE_TEST_ROOT + @"\Folder 1");
      IFileStore dest_store = StoreFactory.CreateStore(StoreFactory.Store.CEStore);
      dest_store.InitializeRoot(dest);

      SyncRule rule = new SyncRule(SyncRule.RuleType.SourceOverridesDestination);

      StoreDiffGram d = new StoreDiffGram(src_store, dest_store, rule);

      Assert.AreEqual(9, d.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);
      Assert.AreEqual(0, d.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(0, d.GetByAction(ItemDiffGram.SyncAction.CopyToSource).Count);
      Assert.AreEqual(0, d.GetByAction(ItemDiffGram.SyncAction.DeleteFromDestination).Count);

      src = new CEStoreItem(TestData.CE_TEST_ROOT + @"\Folder S");
      src_store = new CEStore();
      src_store.InitializeRoot(src);

      dest = new CEStoreItem(TestData.CE_TEST_ROOT + @"\Folder D");
      dest_store = new CEStore();
      dest_store.InitializeRoot(dest);

      rule = new SyncRule(SyncRule.RuleType.SourceOverridesDestination);

      d = new StoreDiffGram(src_store, dest_store, rule);

      Assert.AreEqual(2, d.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);
      Assert.AreEqual(3, d.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(0, d.GetByAction(ItemDiffGram.SyncAction.CopyToSource).Count);
      Assert.AreEqual(2, d.GetByAction(ItemDiffGram.SyncAction.DeleteFromDestination).Count);

      rule.Rule = SyncRule.RuleType.SyncBothWays;

      d = new StoreDiffGram(src_store, dest_store, rule);

      Assert.AreEqual(2, d.GetByAction(ItemDiffGram.SyncAction.Unchanged).Count);
      Assert.AreEqual(2, d.GetByAction(ItemDiffGram.SyncAction.CopyToDestination).Count);
      Assert.AreEqual(3, d.GetByAction(ItemDiffGram.SyncAction.CopyToSource).Count);
      Assert.AreEqual(0, d.GetByAction(ItemDiffGram.SyncAction.DeleteFromDestination).Count);
    }
  }
}